#include "STM32F10x.h"


void USART2_Init (void) 
{
  int i;

	//RCC->APB1ENR |= RCC_APB1ENR_USART2EN;
  RCC->APB2ENR |= RCC_APB2ENR_AFIOEN | RCC_APB2ENR_IOPDEN;
	
	// Remap USART2 to PD5, PD6
	AFIO->MAPR |= AFIO_MAPR_USART2_REMAP;
	
	// PD5 - tx, PD6 - rx
	GPIOD->CRL   &= ~(0xFF <<  20);
  GPIOD->CRL   |=  (0xB <<  20) | (0x4 << 24) | (0x3 <<  28);
  
	// Enable USART2 clk
	RCC->APB1ENR |= RCC_APB1ENR_USART2EN;
	
	
	// Set baudrate to 115200 (NOTE: 14.7456 MHz resonator is used!!!)
  USART2->BRR   = 0x200;                 
	
	// Configure USART2
  USART2->CR1   = ((   1UL <<  2) |       // Enable RX
                   (   1UL <<  3) |       // Enable TX
                   (   0UL << 12) );      // 1 start bit, 8 data bits
  USART2->CR2   = 0x0000;                 // 1 stop bit
  USART2->CR3   = 0x0000;                 // No flow control
  for (i = 0; i < 0x1000; i++) __NOP();   // Avoid unwanted output

  USART2->CR1 |= USART_CR1_UE;      			// Enable USART
}


int SendChar (int ch)  
{

  while (!(USART2->SR & USART_SR_TXE));
  USART2->DR = (ch & 0x1FF);
  return (ch);
}


int GetChar(void)  
{
  while (!(USART2->SR & USART_SR_RXNE));
  return ((int)(USART2->DR & 0x1FF));
}


void SystemInit()
{
	FLASH->ACR |= FLASH_ACR_PRFTBE;
  /* Flash 0 wait state */
  FLASH->ACR = (FLASH->ACR & ~FLASH_ACR_LATENCY) | FLASH_ACR_LATENCY_1;


	// Enable HSE 
	RCC->CR |= RCC_CR_HSEON;
	while (RCC->CR & RCC_CR_HSERDY == 0);

  	// PREDIV1 source = HSE
	RCC->CFGR &= ~RCC_CFGR_PPRE1;

	// PLLSRC = PREDIV1
	RCC->CFGR |= RCC_CFGR_PLLSRC;

	// PLL = PREDIV1 * 4 => SYSCLK = 58.982400 MHz
	RCC->CFGR = (RCC->CFGR & ~RCC_CFGR_PLLMULL) | RCC_CFGR_PLLMULL_1;
	
	//PLL enable
	RCC->CR |= RCC_CR_PLLON;
	while ((RCC->CR & RCC_CR_PLLRDY) == 0);

    RCC->CFGR = (RCC->CFGR & ~RCC_CFGR_SW) | RCC_CFGR_SW_PLL;
	while((RCC->CFGR & RCC_CFGR_SWS) != RCC_CFGR_SWS_PLL);

	// Joy OK - floating input
	RCC->APB2ENR |= RCC_APB2ENR_IOPCEN;
	GPIOC->CRL   &= ~(0xFUL <<  28);
  GPIOC->CRL   |=  (0x4 << 28);
	
	// LED pin (PB0)
	RCC->APB2ENR |= RCC_APB2ENR_IOPBEN;
	GPIOB->CRL   &= ~(0xF);
  GPIOB->CRL   |=  (0x3);
	GPIOB->ODR |= 1;
	
	// PD7 - PWRKEY, PP output
	RCC->APB2ENR |= RCC_APB2ENR_IOPDEN;
	GPIOD->CRL   &= ~(0xFUL <<  28);
  GPIOD->CRL   |=  (0x3 <<  28);
	GPIOD->ODR	 &= ~(1 << 7);
}






void M10_SendCmd(char * cmd, char * res)
{
		int c;

		while (*cmd != 0)
		{
				SendChar(*cmd);
				cmd++;
		}
		
		if (res != 0)
		{
				GetChar();GetChar();
				do {
						c = GetChar();
						*res = c;
						res++;
				} while (c != 0xD);
				*res = 0;
				GetChar();
		}
}

void del(int d)
{
		int i, j;
		for (i = 0; i < d; i++)
				for (j = 0; j < 0x7FFFF; j++);
}

void M10_Init()
{
		unsigned int i;
		char res[128];
		
		del(60);	
		
		// PWRKEY = 1
		GPIOD->ODR |= (1 << 7);
	
		// LED on
		GPIOB->ODR &= ~1;
	
		// Wait for Quectel M10 to start
	  del(60);
		
		for (i=0; i < 3; i++)
	  {
	      M10_SendCmd("AT\x0D", &res[0]);	
				del(3);
		}
		
		// LED off
		GPIOB->ODR |= 1;
		
		// Echo off
		M10_SendCmd("ATE0\x0D", 0);
}

void M10_SendSMS(char * telephone, char * message)
{
		char res[128], cmd[256];
	
		// LED on
		GPIOB->ODR &= ~1;
	
		// Set text mode
		M10_SendCmd("AT+CMGF=1\x0D", &res[0]);
	
		// Send SMS message
		sprintf(cmd, "AT+CMGS=\"%s\"\x0D", telephone);
	  M10_SendCmd(cmd, 0);		
	  sprintf(cmd, "%s\x1A", message);
	  M10_SendCmd(cmd, &res[0]);
	  
		// LED off
		GPIOB->ODR |= 1;
}

int main()
{
	USART2_Init();
	M10_Init();
	while(1)
	{
			if ((GPIOC->IDR & (1 << 7)) == 0 )
			{
					M10_SendSMS("+48503194325", "Quectel M10 test...");
			}
	}
}
